/*
  ========================================================================

  FILE:                 sqlstdlib.h
  
  SERVICES:             Standard SQL UDF Library

  GENERAL DESCRIPTION:  Implements a library of standard user-defined
                        functions for the BREW SQL Extension 1.1 and
                        later.

  ========================================================================
  ========================================================================
    
               Copyright  1999-2007 QUALCOMM Incorporated 
                     All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
    
  ========================================================================
  ========================================================================
*/
#ifndef __SQLSTDLIB_H__
#define __SQLSTDLIB_H__

// Lib mimetype
#define SQLSTDLIB_MIMETYPE                   "stdlib"

// User-defined function names
#define BSTDLIB_ACTIVEAPPLET_NAME            "BSTDLIB_ActiveApplet"
#define BSTDLIB_GETTIMEMS_NAME               "BSTDLIB_GETTIMEMS"
#define BSTDLIB_GETUPTIMEMS_NAME             "BSTDLIB_GETUPTIMEMS"
#define BSTDLIB_GETUTCSECONDS_NAME           "BSTDLIB_GETUTCSECONDS"
#define BSTDLIB_ISDAYLIGHTSAVINGSTIME_NAME   "BSTDLIB_IsDaylightSavingsTime"
#define BSTDLIB_ISTIMEVALID_NAME             "BSTDLIB_ISTIMEVALID"
#define BSTDLIB_LOCALTIMEOFFSET_NAME         "BSTDLIB_LOCALTIMEOFFSET"
#define BSTDLIB_NOTIFY_NAME                  "BSTDLIB_Notify"
#define BSTDLIB_POSTEVENTEX_NAME             "BSTDLIB_PostEventEx"
#define BSTDLIB_POSTURL_NAME                 "BSTDLIB_PostURL"

/*==============================================================================
  DATA STRUCTURE DOCUMENTATION
================================================================================

SQLSTDLIB Interface and Class IDs

Description:
   Each SQL user defined function library is identified by a unique ID.  These 
   IDs are used to distinguish between various types of objects.  Some SQL user
   defined function libraries also identify themselves via a mimetype, allowing
   BREW applications to query the system for particular library types.

   Applications will typically use the ISQLFunc library to load user-defined
   function libraries by their classid or mimetype.  User defined function
   libraries may be created directly using ISHELL_CreateInstance() as well.
   
   SQLSTDLIB recognizes the following class IDs, interface IDs and mimetypes.

Definition:

Identifier              Definition
----------              -------------------------------------------------------
AEEIID_SQLUDFLIB:       Identifies the ISQLUDFLib interface
AEECLSID_SQLSTDLIB:     Identifies the SQLSTDLIB user-defined function library
"x-qc-sqludflib/stdlib" Identifies the SQLSTDLIB user-defined function library

Comments:
   None

See Also:
   ISHELL_CreateInstance()
   ISQLFUNC_LoadLibrary()
   ISQLFUNC_LoadLibraryByClassId()
   ISQLFUNC_LoadLibraryByMimetype()

================================================================================
 
SQLSTDLIB UDF Names

Description:
   The functions defined in the SQLSTDLIB user defined function libary are
   called directly from SQL queries.  Queries are nothing more than strings 
   that are built using SNPRINTF() or an equivalent passed to ISQL for 
   exectution.  The SQLSTDLIB provides the following constants for applications
   to use when building queries to invoke the functions it implements:

   
Definition:
===pre>
   Definition                             Value
   ----------------------                 ------------------------------------
   BSTDLIB_ACTIVEAPPLET_NAME              "BSTDLIB_ActiveApplet"
   BSTDLIB_GETTIMEMS_NAME                 "BSTDLIB_GETTIMEMS"
   BSTDLIB_GETUPTIMEMS_NAME               "BSTDLIB_GETUPTIMEMS"
   BSTDLIB_GETUTCSECONDS_NAME             "BSTDLIB_GETUTCSECONDS"
   BSTDLIB_ISDAYLIGHTSAVINGSTIME_NAME     "BSTDLIB_IsDaylightSavingsTime"
   BSTDLIB_ISTIMEVALID_NAME               "BSTDLIB_ISTIMEVALID"
   BSTDLIB_LOCALTIMEOFFSET_NAME           "BSTDLIB_LOCALTIMEOFFSET"
   BSTDLIB_NOTIFY_NAME                    "BSTDLIB_Notify"
   BSTDLIB_POSTEVENTEX_NAME               "BSTDLIB_PostEventEx"
   BSTDLIB_POSTURL_NAME                   "BSTDLIB_PostURL"
===/pre>

Members:
   None
   
Comments:
   None

See Also:
   None

================================================================================
 INTERFACE DOCUMENTATION
================================================================================

BREW SQL Standard Library

Description: 
   Certain functions are universally useful to applications to implement 
   database change notifications. Combined with triggers, they may be used to 
   implement notifications for BREW applications. This standard functionality 
   is implemented and included in sqlstdlib, an external user-defiend library 
   implementation that ships alongside the BREW SQL Extension.

   To use the functions implemented in the SQLSTDLIB user define function
   library extension, applications must first load the udf library by calling
   ISQLFUNC_LoadLibrary() or one of its variants.  This loads the library into
   memory and registers the functions implemented within with a particular 
   instance of ISQL.  Once loaded, the functions described here can be called
   directly from queries send to ISQL_Exec().

===H2>
Required header files
===/H2>
===pre>
sqlstdlib.h
===/pre>

See Also:
   ISQLFUNC_LoadLibrary()
   ISQLFUNC_LoadLibraryByClassId()
   ISQLFUNC_LoadLibraryByMimetype()
   
==========================================================================

BSTDLIB_ActiveApplet()

Description:
   What application is modifying the database? This UDF wraps BREW's 
   ISHELL_ActiveApplet() call.

Definition:
   int BSTDLIB_ActiveApplet(void);

Parameters:
===pre>
   none
===/pre>

Return Value:
===pre>
   The class id of the application that owns the ISQL object that has 
   executed the query.
===/pre>

Comments:
   This function can be invoked using the follows SQL query:
===pre>
   SELECT BSTDLIB_ActiveApplet();
===/pre>

==========================================================================

BSTDLIB_GETTIMEMS()

Description:
   Return the current time in MS. This UDF wraps BREW's GETTIMEMS() call.

Definition:
   int BSTDLIB_GETTIMEMS(void);

Parameters:
===pre>
   none
===/pre>

Return Value:
===pre>
   This method returns the current time of day in milliseconds. 
===/pre>

Comments:
   This function can be invoked using the follows SQL query:
===pre>
   SELECT BSTDLIB_GETTIMEMS();
===/pre>
   Note that the value returned by this call depends on the device's 
   current time of day value. This value is obtained from the base 
   station. As such this value may change dramatically when the phone 
   first acquires system coverage.

==========================================================================

BSTDLIB_GETUPTIMEMS()

Description:
   Return the device's uptime. This UDF wraps BREW's GETUPTIMEMS() call.

Definition:
   int BSTDLIB_GETUPTIMEMS(void);

Parameters:
===pre>
   none
===/pre>

Return Value:
===pre>
   32-bit millisecond value from the time the device was powered up.
===/pre>

Comments:
   This function can be invoked using the follows SQL query:
===pre>
   SELECT BSTDLIB_GETUPTIMEMS();
===/pre>

==========================================================================

BSTDLIB_GETUTCSECONDS()

Description:
   Return CDMA time, the number of seconds since 1/6/80 00:00:00 GMT. 
   This UDF wraps BREW's GETUTCSECONDS() call.

Definition:
   int BSTDLIB_GETUTCSECONDS(void);

Parameters:
===pre>
   none
===/pre>

Return Value:
===pre>
   The number of seconds since 1/6/80 00:00:00 GMT.
===/pre>

Comments:
   This function can be invoked using the follows SQL query:
===pre>
   SELECT BSTDLIB_GETUTCSECONDS();
===/pre>

==========================================================================

BSTDLIB_IsDaylightSavingsTime()

Description:
   This UDF wrapts BREW's LOCALTIMEOFFSET() call, and returns whether 
   daylight savings time is in effect.

Definition:
   int BSTDLIB_IsDaylightSavingsTime(void);

Parameters:
===pre>
   none
===/pre>

Return Value:
===pre>
   (0) if daylight savings time is not currently in effect.
   (1) if daylight savings time is currently in effect.
===/pre>

Comments:
   This function can be invoked using the follows SQL query:
===pre>
   SELECT BSTDLIB_IsDaylightSavingsTime();
===/pre>

==========================================================================

BSTDLIB_ISTIMEVALID()

Description:
   This UDF wrapts BREW's ISTIMEVALID() macro:

   #define AEE_SECS_PER_YEAR  31536000L    // one year in seconds 
   #define ISTIMEVALID()      (GETUTCSECONDS() > AEE_SECS_PER_YEAR)

Definition:
   int BSTDLIB_ISTIMEVALID(void);

Parameters:
===pre>
   none
===/pre>

Return Value:
===pre>
   (0) if the current time is invalid, something earlier than 1981
   (1) if the current time is valid, 1981 or later
===/pre>

Comments:
   This function can be invoked using the follows SQL query:
===pre>
   SELECT BSTDLIB_ISTIMEVALID();
===/pre>

==========================================================================

BSTDLIB_LOCALTIMEOFFSET()

Description:
   Return the local time offset. This UDF wrapts BREW's LOCALTIMEOFFSET() 
   call.

Definition:
   int BSTDLIB_LOCALTIMEOFFSET(void);

Parameters:
===pre>
   none
===/pre>

Return Value:
===pre>
   Returns the local timezone offset from UTC in seconds.
===/pre>

Comments:
   This function can be invoked using the follows SQL query:
===pre>
   SELECT BSTDLIB_LOCALTIMEOFFSET();
===/pre>

==========================================================================

BSTDLIB_Notify()

Description:
   Send a notification event. This UDF wraps BREW's ISHELL_Notify() call
   and notifies all applications registered for events from the either the
   notifier specified in the cal, or the AEECLSID_SQLSTDLIBNOTIFIER
   notifier (if none is specified).  This function takes a mask that can 
   describe the type of notification being sent as well as a string as the 
   notify data.

   Called from a trigger, this function allows applications to be started
   up and notified when a particular change in a database occurs.  Apps
   will need to agree on the values of the mask and notify data to
   coordinate properly.

Definition:
   int BSTDLIB_Notify(AEECLSID clsType, int mask, text notifyData);

Parameters
===pre>
   clsType - class that issued the notify, or 0 to use  
             AEECLSID_SQLSTDLIBNOTIFIER. 
   mask - a 32 bit integer, passed as-is to ISHELL_Notify
   notifyData - a string that's passed to ISHELL_Notify as the payload
===/pre>

Return Value
===pre>
   SUCCESS (0) if the ISHELL_Notify() call succeeds
   Error (non-zero) otherwise.
===/pre>

Comments:
   This function can be invoked using the follows SQL query:
===pre>
   SELECT BSTDLIB_Notify(0, 1, 'addressbook:record changed');
===/pre>
   Applications may implement their own INotifier class and have this
   function send a notification using it by specifying its classId in
   the clsType parameter.  In this case, BSTDLIB_Notify will send a
   notification event to all entities registered with the custom
   notifier.
   
   Applications may also rely on the built-in AEECLSID_SQLSTDLIBNOTIFIER 
   INotifier implementation as well by passing in zero (0) for clsType.
   In this case, BSTDLIB_Notify will send notifications to all entities
   registered for notifications from the AEECLSID_SQLSTDLIBNOTIFIER 
   notifier.  This provides a complete generic notification system, but
   may not be as efficient as a custom notifier that only a handful of
   applications are listening to.
   
==========================================================================

BSTDLIB_PostEventEx()

Description:
   Post an event to another application. This UDF wraps BREW's 
   ISHELL_PostEventEx() call.

Definition:
   int BSTDLIB_PostEvent(uint16 wFlags, 
                         AEECLSID clsApp, 
                         uint16 evt, 
                         uint16 wp, 
                         uint32 dwp);

Parameters:
===pre>
   wFlags - EVTFLG_UNIQUE (1) to enforce unique events or 
                          (0) to allow multiple events to be queued.
   clsApp - class id of the application to send the event to.
   evt - the event to send. A 16 bit unsigned int.
   wp - a 16 bit unsigned int to send with the event.
   dwp - a 32 bit unsigned int to send with the event.
===/pre>

Return Value:
===pre>
   SUCCESS (0) if call executes successfully 
   Error (non-zero) if the event could not be queued.
===/pre>

Comments:
   This function can be invoked using the follows SQL query:
===pre>
   SELECT BSTDLIB_PostEvent(<uint16 wflags>,
                            <AEECLSID classid>,
                            <uint16 evt>,
                            <uint16 wp>,
                            <uint32 dwp>);
===/pre>
   This would send a "evt" event to the application identified with a 
   class id of "classid". "wp" and "dwp" will be passed into the 
   application's HandleEvent() routine. If the caller sets <wFlags> to 
   0, many events of the same type may be queued for the same application.
   
   If the caller sts <wFlags> to EVTFLG_UNIQUE (ie, 1), then the event is 
   posted only if there are no other events queued for the target application.

   If this UDF is called from a trigger, it is recommended to NOT use 
   EVTFLG_UNIQUE. Triggers may fire multiple times and post multiple events 
   before the target application gets any time to process its event queue.

==========================================================================

BSTDLIB_PostURL()

Description:
   Post a URL. This UDF wraps BREW's ISHELL_PostURL() call. This function 
   posts an asynchronous event, EVT_APP_POST_URL, to the applet associated 
   with the URL's scheme.

Definition:
   int BSTDLIB_PostURL(text url);

Parameters
===pre>
   url - the url to post.
===/pre>

Return Value
===pre>
   SUCCESS (0) if ISHELL_PostURL() call succeeds
   Error (non-zero) otherwise.
===/pre>

Comments:
   This function can be invoked using the follows SQL query:
===pre>
   SELECT BSTDLIB_PostURL('scheme:event');
===/pre>
========================================================================*/

#endif // __SQLSTDLIB_H__